/*
Phi-2 shield for Arduino
Program title: Phi clock version 5 
----------------------------------------
Instructions:
0. Please download phi_interfaces library (http://liudr.wordpress.com/phi_interfaces/) and unzip the files and example subfolders in arduino\sketchbooks\libraries\phi_interfaces\
1. Please download phi_prompt library (http://liudr.wordpress.com/phi_prompt/) and unzip the files and example subfolders in arduino\sketchbooks\libraries\phi_prompt\
2. Please download phi_big_font library (http://liudr.wordpress.com/phi_big_font/) and unzip the three files and example subfolders in arduino\sketchbooks\libraries\phi_big_font\
3. Please set the size of you display with "#define lcd_rows" and "#define lcd_columns" so the program will automatically adjust to your display size.
4. You need to set the clock before you can use it. Uncomment this line "#define setRTC" among the #define lines.
5. Update the time in setup() to the current time.
6. Upload the sketch to arduino to set the time.
7. Then comment the line "#define setRTC" and recompile and upload to arduino.
8. If you don't do step 4, the clock will be reset every time arduino resets.
----------------------------------------
Programmed by Dr. John Liu
Revision: 08/26/2012
Free software for educational and personal uses.
No warrantee!
Commercial use without authorization is prohibited.
Find details of the Phi-1 shield, Phi-2 shield, and Phi-prompt or contact Dr. Liu at
http://liudr.wordpress.com/phi-1-shield/
http://liudr.wordpress.com/phi-2-shield/
http://liudr.wordpress.com/phi_interfaces/
http://liudr.wordpress.com/phi_prompt/
All rights reserved.
----------------------------------------
List of functions:
* Display real time clock with Year month, date, day of week, hour, minute, and second.
* Four individual alarms (change Max_alarms to get more alarms)
* Alarms are stored on Arduino EEPROM and won't be lost when the power is out.
* Alarms Daily, Weekday, Weekend, Once, Off.
* Adjustable alarms. Press B to choose alarm and adjust the time.
* Adjustable clock and date. Press A to adjust clock and date.
* TODO - Add snooze function.
* TODO - Add special function on the alarm to drive outputs other than the buzzer and LED. You can be creative!
----------------------------------------
Thanks:

RTC created by matt.joyce@gmail.com

EEPROM Author: hkhijhe

Dependent libraries:
Phi_prompt version 1.0
Phi_interfaces version 1.0
Phi_big_font version 1.0

Updates:
 * 8/26/2012: I corrected the bug Andrew found about the "ONCE" alarm not turned off in EEPROM settings.
 * I rewrote the code to save the status of a "ONCE" alarm to EEPROM as "OFF" once it is triggered. This was made possible with the addition of method save_alarm().
 * Now the set alarm function in the Example_menu also calls this save_alarm method.
*/

/*
Modifications made by Dantana
-Switched from previous DS1307_1 library to DS1307 by D. Sjunnesson (DS1307_1 had compatibility problems with 1.0)
-Removed L and R buttons (5 and 11), using those digital outs for PWM gradients of lights on
-Changed intro screen
-Changed alarm so that it activates LEDs instead of buzze

*/
#define phi_2_shield
#include <LiquidCrystal.h>
#include <Wire.h>
#include <stdio.h>
#include <avr/pgmspace.h>
#include <EEPROM.h>
#include <phi_interfaces.h>
#include <phi_prompt.h>
#include <phi_big_font.h>
#include "DS1307.h" 
#include "alarm_clock.h"

#define lcd_rows 2
#define lcd_columns 16
//#define setRTC //comment this line to have it not reset the clock every time
//Phi-2 shield definitions

//Phi-2 shield buttons and channel pin assignments
#define buzzer A2
#define total_buttons 5
#define btn_u 5
#define btn_d 10
//#define btn_l 11
#define btn_r 4 //reassigned to btn_u, was 5
#define btn_b A0
#define btn_a A1

#define LED_1 A3 //was 11
#define LED_2 11

//Phi-2 shield LCD pin setting
#define LCD_RS 8
#define LCD_EN 9
#define LCD_D4 7
#define LCD_D5 6
#define LCD_D6 2
#define LCD_D7 3

PROGMEM prog_char msg_00[]="Sunrise Alarm Clock";
PROGMEM prog_char msg_01[]="Sunrise Demo:   Press Enter";

PROGMEM prog_char month_00[]="JAN";
PROGMEM prog_char month_01[]="FEB";
PROGMEM prog_char month_02[]="MAR";
PROGMEM prog_char month_03[]="APR";
PROGMEM prog_char month_04[]="MAY";
PROGMEM prog_char month_05[]="JUN";
PROGMEM prog_char month_06[]="JUL";
PROGMEM prog_char month_07[]="AUG";
PROGMEM prog_char month_08[]="SEP";
PROGMEM prog_char month_09[]="OCT";
PROGMEM prog_char month_10[]="NOV";
PROGMEM prog_char month_11[]="DEC";
PROGMEM const char *month_items[]= {month_00,month_01,month_02,month_03,month_04,month_05,month_06,month_07,month_08,month_09,month_10,month_11};

PROGMEM prog_char dow_00[]="SUN";
PROGMEM prog_char dow_01[]="MON";
PROGMEM prog_char dow_02[]="TUE";
PROGMEM prog_char dow_03[]="WED";
PROGMEM prog_char dow_04[]="THU";
PROGMEM prog_char dow_05[]="FRI";
PROGMEM prog_char dow_06[]="SAT";
PROGMEM const char *dow_items[]= {dow_00,dow_01,dow_02,dow_03,dow_04,dow_05,dow_06};

PROGMEM prog_char alarm_00[]="Off";
PROGMEM prog_char alarm_01[]="Daily";
PROGMEM prog_char alarm_02[]="Weekday";
PROGMEM prog_char alarm_03[]="Weekend";
PROGMEM prog_char alarm_04[]="Once";
PROGMEM const char *alarm_items[]= {alarm_00,alarm_01,alarm_02,alarm_03,alarm_04};

PROGMEM prog_char lcd_chr0[]={64,64,64,64,64,64,64,64,0};// 0
PROGMEM prog_char lcd_chr1[]={64,64,64,64,64,31,31,31,0};//1
PROGMEM prog_char lcd_chr2[]={64,64,64,31,31,64,64,64,0};//2
PROGMEM prog_char lcd_chr3[]={64,64,64,31,31,31,31,31,0};//3 {64,64,64,31,31,64,31,31,0};
PROGMEM prog_char lcd_chr4[]={31,31,31,64,64,64,64,64,0};//4
PROGMEM prog_char lcd_chr5[]={31,31,64,64,64,31,31,31,0};//5
PROGMEM prog_char lcd_chr6[]={31,31,31,31,31,64,64,64,0};//6 {31,31,64,31,31,64,64,64,0}
PROGMEM prog_char lcd_chr7[]={31,31,31,31,31,31,31,31,0};//7 {31,31,64,31,31,64,31,31,0}
PROGMEM const char *chr_item[] = {lcd_chr0, lcd_chr1, lcd_chr2, lcd_chr3, lcd_chr4, lcd_chr5, lcd_chr6, lcd_chr7};// The following lines instantiates a button group to control 6 buttons

byte pins[]={btn_u,btn_d,/*btn_l,*/btn_r, btn_b,btn_a}; // The digital pins connected to the 6 buttons.
char mapping[]={'U','D',/*'L',*/'R','B','A'}; // This is a list of names for each button.
phi_button_groups my_btns(mapping, pins, total_buttons);

// This serial keypad is for debugging.
phi_serial_keypads debug_keypad(&Serial,115200); 

// The following sets up function keys for phi_prompt library
char up_keys[]={"U"}; ///< All keys that act as the up key are listed here.
char down_keys[]={"D"}; ///< All keys that act as the down key are listed here.
//char left_keys[]={"L"}; ///< All keys that act as the left key are listed here.
char right_keys[]={"R"}; ///< All keys that act as the right key are listed here.
char enter_keys[]={"B"}; ///< All keys that act as the enter key are listed here.
char escape_keys[]={"A"}; ///< All keys that act as the escape key are listed here.
char * function_keys[]={up_keys,down_keys,/*left_keys,*/right_keys,enter_keys,escape_keys}; ///< All function key names are gathered here fhr phi_prompt.

// The following adds all available keypads as inputs for phi_prompt library
multiple_button_input * keypads[]={&my_btns,&debug_keypad,0};

LiquidCrystal lcd(LCD_RS,LCD_EN,LCD_D4,LCD_D5,LCD_D6,LCD_D7); // Create the lcd object

int rtc[7];
//DS1307 RTC=DS1307 (); // Create RTC object, already created with this DS1307 library
alarm_clock clock1(false); // Create an alarm clock

void setup()
{
  Serial.begin(9600);
  Serial.println("Welcome to the machine.");
  byte ch_buffer[10]; // This buffer is required for custom characters on the LCD.
  lcd.begin(lcd_columns, lcd_rows);
  Wire.begin(); // initialize wire
  init_phi_prompt(&lcd,keypads,function_keys, lcd_columns, lcd_rows, '~'); // Supply the liquid crystal object, input keypads, and function key names. Also supply the column and row of the lcd, and indicator as '>'. You can also use '\x7e', which is a right arrow.
  init_big_font(&lcd);

// Set the two channels to output to drive the buzzer and LED.
  pinMode(buzzer,OUTPUT);
  digitalWrite(buzzer,LOW);
  pinMode(LED_1, OUTPUT);   // Set digital pin 11 as OUTPUT pin
  digitalWrite(LED_1,LOW);
  pinMode(LED_2, OUTPUT);   // Set digital pin 11 as OUTPUT pin
  digitalWrite(LED_2,LOW);

#ifdef setRTC
// Set/init RTC
  RTC.stop();
  RTC.set(DS1307_SEC,40);
  RTC.set(DS1307_MIN,29);
  RTC.set(DS1307_HR,23);
  RTC.set(DS1307_DOW,4); // value from 1 to 7. User define whether 1 is sun or mon.
  RTC.set(DS1307_DATE,11);
  RTC.set(DS1307_MTH,5);
  RTC.set(DS1307_YR,13);
  RTC.start();

//Set alarms
  EEPROM.write(alarm_EEPROM_storage,7); // Alarm 0 hour
  EEPROM.write(alarm_EEPROM_storage+1,31); // Alarm 0 minute
  EEPROM.write(alarm_EEPROM_storage+2,2); // Alarm 0 frequency M-F

  EEPROM.write(alarm_EEPROM_storage+3,7); // Alarm 1 hour
  EEPROM.write(alarm_EEPROM_storage+4,00); // Alarm 1 minute
  EEPROM.write(alarm_EEPROM_storage+5,2); // Alarm 1 frequency M-F

  EEPROM.write(alarm_EEPROM_storage+6,7); // Alarm 2 hour
  EEPROM.write(alarm_EEPROM_storage+7,10); // Alarm 2 minute
  EEPROM.write(alarm_EEPROM_storage+8,2); // Alarm 2 frequency M-F

  EEPROM.write(alarm_EEPROM_storage+9,9); // Alarm 3 hour
  EEPROM.write(alarm_EEPROM_storage+10,30); // Alarm 3 minute
  EEPROM.write(alarm_EEPROM_storage+11,3); // Alarm 3 frequency Weekend
#else
// Set alarm on the clock from EEPROM.
  clock1.set_alarm(0, EEPROM.read(alarm_EEPROM_storage), EEPROM.read(alarm_EEPROM_storage+1), EEPROM.read(alarm_EEPROM_storage+2));
  clock1.set_alarm(1, EEPROM.read(alarm_EEPROM_storage+3), EEPROM.read(alarm_EEPROM_storage+4), EEPROM.read(alarm_EEPROM_storage+5));
  clock1.set_alarm(2, EEPROM.read(alarm_EEPROM_storage+6), EEPROM.read(alarm_EEPROM_storage+7), EEPROM.read(alarm_EEPROM_storage+8));
  clock1.set_alarm(3, EEPROM.read(alarm_EEPROM_storage+9), EEPROM.read(alarm_EEPROM_storage+10), EEPROM.read(alarm_EEPROM_storage+11));
#endif

  show_credit();
}

void loop()
{
  top_menu(); // See Example_menu.pde
}

void show_credit()
{
  // Display credits
  phi_prompt_struct myLongMsg;

  lcd.clear();
  lcd.noBlink();
  myLongMsg.ptr.msg_P=msg_00; // Assign the address of the text string to the pointer.
  myLongMsg.low.i=0; // Default text starting position. 0 is highly recommended.
  myLongMsg.high.i=strlen_P(msg_00); // Position of the last character in the text string, which is size of the string - 1.
  myLongMsg.step.c_arr[0]=lcd_rows; // rows to auto fit entire screen
  myLongMsg.step.c_arr[1]=lcd_columns; // one col list
  myLongMsg.col=0; // Display the text area starting at column 0
  myLongMsg.row=0; // Display the text area starting at row 0
  myLongMsg.option=0; // Option 0, display classic message, option 1, display message with scroll bar on right.

  text_area_P(&myLongMsg);
}

void show_demo()
{
  // Display credits
  phi_prompt_struct myLongMsg;

  lcd.clear();
  lcd.noBlink();
  myLongMsg.ptr.msg_P=msg_01; // Assign the address of the text string to the pointer.
  myLongMsg.low.i=0; // Default text starting position. 0 is highly recommended.
  myLongMsg.high.i=strlen_P(msg_01); // Position of the last character in the text string, which is size of the string - 1.
  myLongMsg.step.c_arr[0]=lcd_rows; // rows to auto fit entire screen
  myLongMsg.step.c_arr[1]=lcd_columns; // one col list
  myLongMsg.col=0; // Display the text area starting at column 0
  myLongMsg.row=0; // Display the text area starting at row 0
  myLongMsg.option=0; // Option 0, display classic message, option 1, display message with scroll bar on right.

  text_area_P(&myLongMsg);
  
  int key_press=0;
  int duty = 0;
  int steps = 64;
  int sunrisespeed = 20;
  int i,j;
  int k=0;
  int remainder, div, iremainder, idiv;

  int lookup[64] = {1,2,4,6,9,12,16,20,25,30,36,
            42,49,56,64,72,81,90,100,110,121,132,
            144,156,169,182,196,210,225,240,256,272,289,
            306,324,342,361,380,400,420,441,462,484,506,
            529,552,576,600,625,650,676,702,729,756,784,
            812,841,870,900,930,961,992,992,992};
            
  //64 steps. Time on each step is 5ms * sunrise speed. Sunrise speed=200 means a 1minute sunrise. Sunrise speed=6000 means a 30 minute sunrise.
  //Taken from www.instructables.com/id/Simple-Arduino-Sunrise-Lamp/
  //LED_2 = PWM
 
  for (i=0; i<steps; i++)
  {    
    duty = lookup[i] * 5;
    for (j=0; j<sunrisespeed; j++)
    {   
      digitalWrite(buzzer, HIGH);
      delayMicroseconds(duty);

      digitalWrite(buzzer, LOW);
      delayMicroseconds(5000-duty);
    }
  }
 digitalWrite(buzzer, HIGH);
 duty=0;

  for (i=0; i<steps; i++)
  { 
    duty = lookup[i] * 5;
    for (j=0; j<sunrisespeed; j++)
    {   
      digitalWrite(LED_1, HIGH);
      delayMicroseconds(duty);

      digitalWrite(LED_1, LOW);
      delayMicroseconds(5000-duty);
    }
  }
  digitalWrite(LED_1, HIGH);  
 
  while(key_press!=5&&key_press!=6&&k<300)
  {
    if(k<255)
      analogWrite(LED_2, k);
    else
        digitalWrite(LED_2, HIGH);
    
    digitalWrite(LED_1, HIGH);
    //digitalWrite(LED_2, HIGH);
    digitalWrite(buzzer, HIGH);
    key_press=wait_on_escape(100);
    k++;
  }
  digitalWrite(LED_1, LOW);
  digitalWrite(LED_2, LOW);
  digitalWrite(buzzer, LOW);
}

