/*
Phi-2 shield for Arduino
Program title: Phi clock version 5 
----------------------------------------
Programmed by Dr. John Liu
Revision: 02/28/2012
Free software for educational and personal uses.
No warrantee!
Commercial use without authorization is prohibited.
Find details of the Phi-1 shield, Phi-2 shield, and Phi-prompt or contact Dr. Liu at
http://liudr.wordpress.com/phi-1-shield/
http://liudr.wordpress.com/phi-2-shield/
http://liudr.wordpress.com/phi_interfaces/
http://liudr.wordpress.com/phi_prompt/
All rights reserved.
*/
#include "alarm_clock.h"
#include <EEPROM.h>

void render_RTC(int temp)
{
  int rtc[7];
  char msg[17];
  int user_input;
  phi_prompt_struct myListInput; // This structure stores the main menu.

  RTC.get(rtc,true);

  switch(temp)
  {
    case 0:
    myListInput.ptr.list=(char**)&month_items; // Assign the list to the pointer
    myListInput.low.i=rtc[5]-1; // Default item highlighted on the list
    myListInput.high.i=11; // Last item of the list is size of the list - 1.
    myListInput.width=3; // Length in characters of the longest list item.
    myListInput.col=lcd_columns/2-8; // Display prompt at column 0
    myListInput.row=lcd_rows/2-1; // Display prompt at row 1
    myListInput.option=0; 
    myListInput.step.c_arr[0]=1; // rows to auto fit entire screen
    myListInput.step.c_arr[1]=1; // one col list
    render_list(&myListInput);
    sprintf(msg,"/%02d/%4d/",rtc[4],rtc[6]);
    lcd.print(msg);
    
    myListInput.ptr.list=(char**)&dow_items; // Assign the list to the pointer
    myListInput.low.i=rtc[3]-1; // Default item highlighted on the list
    myListInput.high.i=6; // Last item of the list is size of the list - 1.
    myListInput.col=lcd_columns/2-8+12; // Display prompt at column 0
    render_list(&myListInput);

    lcd.setCursor(lcd_columns/2-4,lcd_rows/2);
    sprintf(msg,"%2d:%02d:%02d",rtc[2],rtc[1],rtc[0]);
    lcd.print(msg);
    break;
    
    case 1:
    sprintf(msg,"%02d%02d",rtc[2],rtc[1]);
    render_big_msg(msg,lcd_columns/2-8,0);
    if ((lcd_rows==4)&&(lcd_columns==20))
    {
      strcpy_P(msg,(char*)pgm_read_word(dow_items+rtc[3]-1));
      sprintf(msg+3,"%02d",rtc[4]);
      render_big_msg(msg,0,2);
    }
    if (rtc[0]%2)
    {
      lcd.setCursor(lcd_columns/2-1,0);
      lcd.write('.');
      lcd.setCursor(lcd_columns/2-1,1);
      lcd.write('.');
    }
    else
    {
      lcd.setCursor(lcd_columns/2-1,0);
      lcd.write(' ');
      lcd.setCursor(lcd_columns/2-1,1);
      lcd.write(' ');
    }
    break;
    default:
    break;
  }
}

alarm_clock::alarm_clock(boolean noo) // This parameter is needed otherwise the compiler won't work with a constructor without parameters.
{
  alarm_is_on=false;
  snooze=10;
}

byte alarm_clock::run()
{
  int rtc[7];
  render_RTC(clock_style);
  RTC.get(rtc,true);
  for (int i=0;i<4;i++)
  {
    if ((alarms[i].hr==rtc[2])&&(alarms[i].mnt==rtc[1])&&(rtc[0]==0)&&within(i,rtc[3]))
    {
      alarm_is_on=true;
    }
  }
  if (alarm_is_on) alarm();
}

byte alarm_clock::set_alarm(byte alarm_num, byte hr, byte mnt, byte dow)
{
  if (alarm_num>=Max_alarms) return 255;
  if (hr>23) return 255;
  if (mnt>59) return 255;
  alarms[alarm_num].hr=hr;
  alarms[alarm_num].mnt=mnt;
  alarms[alarm_num].dow=dow;
  return 0;
}

void alarm_clock::alarm()
{
  //LED_1 11
  int key_press=0;
  int duty = 0;
  int steps = 64;
  int sunrisespeed = 2000;
  int i,j;
  int k=0;
  int remainder, div, iremainder, idiv;

  int lookup[64] = {1,2,4,6,9,12,16,20,25,30,36,
            42,49,56,64,72,81,90,100,110,121,132,
            144,156,169,182,196,210,225,240,256,272,289,
            306,324,342,361,380,400,420,441,462,484,506,
            529,552,576,600,625,650,676,702,729,756,784,
            812,841,870,900,930,961,992,992,992};
            
  //64 steps. Time on each step is 5ms * sunrise speed. Sunrise speed=200 means a 1minute sunrise. Sunrise speed=6000 means a 30 minute sunrise.
  //Taken from www.instructables.com/id/Simple-Arduino-Sunrise-Lamp/
  //LED_2 = PWM
 
  for (i=0; i<steps; i++)
  {    
    duty = lookup[i] * 5;
    for (j=0; j<sunrisespeed; j++)
    {   
      // one pulse of manual PWM
      //digitalWrite(LED_1, HIGH);
      digitalWrite(buzzer, HIGH);
//    key_press=wait_on_escape(duty);
      delayMicroseconds(duty); //while still keeping the timing absolutely correct
//      Serial.println(duty);

      //digitalWrite(LED_1, LOW);
      //digitalWrite(LED_2, LOW);      
      digitalWrite(buzzer, LOW);
//      key_press=wait_on_escape(5000-duty);
      delayMicroseconds(5000-duty);
    }
  }
 digitalWrite(buzzer, HIGH);
 duty=0;

  for (i=0; i<steps; i++)
  { 
    duty = lookup[i] * 5;
    for (j=0; j<sunrisespeed; j++)
    {   
      digitalWrite(LED_1, HIGH);
      delayMicroseconds(duty);

      digitalWrite(LED_1, LOW);
      delayMicroseconds(5000-duty);
    }
  }
  digitalWrite(LED_1, HIGH);  
 
  while(1)
  {
    if(k<255)
      analogWrite(LED_2, k);
    else
        digitalWrite(LED_2, HIGH);
    
    digitalWrite(LED_1, HIGH);
    //digitalWrite(LED_2, HIGH);
    digitalWrite(buzzer, HIGH);
    key_press=wait_on_escape(1500);
    
    if(key_press==5||key_press==6)
    {
      digitalWrite(LED_1, LOW);
      digitalWrite(LED_2, LOW);
      digitalWrite(buzzer, LOW);
      alarm_is_on=false;
      return;
    }
    k++;
  }
  /*
  int temp1;
  for (int i=0;i<4;i++)
  {
    tone(buzzer,770);
    temp1=wait_on_escape(75);
    if ((temp1==5)||(temp1==6))
    {
      noTone(buzzer);
      alarm_is_on=false; // Turn off alarm
      return;
    }
    noTone(buzzer);
    temp1=wait_on_escape(75);
    if ((temp1==5)||(temp1==6))
    {
      noTone(buzzer);
      alarm_is_on=false; // Turn off alarm
      return;
    }
  }
  temp1=wait_on_escape(150);
  if ((temp1==5)||(temp1==6))
  {
    noTone(buzzer);
    alarm_is_on=false; // Turn off alarm
    return;
  }*/
}

boolean alarm_clock::within(byte ala, byte dow) // This function checks if the day of the week is within alarm clock's trigger setting.
{
  switch(alarms[ala].dow)
  {
    case 0: // Alarm is off
    return false;
    break;
    
    case 1: // Alarm is on daily
    return true;
    break;
    
    case 2: // Alarm is on M-F
    if ((dow>1)&&(dow<7)) return true;
    else return false;
    break;
    
    case 3: // Alarm is on Weekend
    if ((dow==1)||(dow==7)) return true;
    else return false;
    
    case 4: // Alarm is on once so set it to off and return true
    alarms[ala].dow=0;
    save_alarm(ala); //Save the status of this once clock to EEPROM so it stays off after on triggering.
    return true;
    break;
    
    default:
    return false;
    break;
  }
}

void alarm_clock::turn_on(byte alarm_num) // This function is not used.
{
  if (alarm_num<Max_alarms) alarms[alarm_num].on_off=true;
} 

void alarm_clock::turn_off(byte alarm_num) // This function is not used.
{
  if (alarm_num<Max_alarms) alarms[alarm_num].on_off=false;
} 

void alarm_clock::save_alarm(byte ala) // This function saves the alarm setting of the alarm ala to the ATMEGA328 EEPROM.
{
  EEPROM.write(alarm_EEPROM_storage+3*ala,alarms[ala].hr); // Alarm hour
  delay(100);

  EEPROM.write(alarm_EEPROM_storage+3*ala+1,alarms[ala].mnt); // Alarm minute
  delay(100);

  EEPROM.write(alarm_EEPROM_storage+3*ala+2,alarms[ala].dow); // Alarm frequency M-F
  delay(100);
} 

